/****** axuucp/ax-rnews ******************************************************
*
*   NAME
*	ax-rnews - Perform relaynews: import a news batch to AXsh news system
*
*   SYNOPSIS
*	rx ax-rnews.rexx
*
*   DESCRIPTION
*	This script reads an uncompressed news batch from stdin and posts the
*	articles into its groups.  Artiles are expected to be separated by a
*	line of the following format:
*
*		#! rnews xxxxxxxx
*
*	with 'xxxxxxxx' representing the size of the following article.
*	To be more precise: 'xxxxxxxx' must be the number of lines puls the
*	number of characters in all lines, beause a line break has to be
*	counted as one character, even if a LF and a CR are used.
*
*	The contents of the 'Newsgroups:' field will be used to find the 
*	groups in which the article is about to be posted.  If the group does
*	exist on your local system then it will be added via addgroup.
*
*   AUTHOR
*	Tobias Ferber <tf@ganymed.hall.sub.org>
*
******************************************************************************
*
*/

axnews   = axconfig("news")
history  = axnews || "history"
tempfile = "T:ax-rnews." || pragma('Id')
in       = 'STDIN'

articles = 0
cross    = 0

groups   = ""
msgid    = ""
hsize    = 0   /* article size (chars + lines) as propagated in the separator */
lines    = 0
chars    = 0

/**/

str = readln(in)
state = 'start'

do until eof(in)
  select
    when state='start' then do
      if left(str,8) = '#! rnews' then do
        parse var str '#! rnews ' hsize
        hsize= strip(hsize,'L','0')
        lines= 0; chars= 0; groups=''; msgid=''
        state= 'header'
        call open('out',tempfile,'W')
        end
      else do
        if words(str) > 0 then say '"#! rnews" expected in line 1'
        exit
        end
      end

    when state='header' then do
      chars = chars + length(str)
      lines = lines + 1
      if words(str) > 0 then select
        when upper(left(str,12)) = 'NEWSGROUPS: ' then groups= substr(str,13)
        when upper(left(str,12)) = 'MESSAGE-ID: ' then msgid= substr(str,13)
        otherwise nop
        end
      else state='body'
      end

    when state='body' then do
      if left(str,8) = '#! rnews' then do
        if chars+lines ~= hsize then say tempfile': article size mismatch (#! rnews' hsize '!=' chars+lines')'
        call close('out')
        if postnews(tempfile,msgid,groups) > 1 then cross=cross+1
        articles= articles+1
        state= 'start'
        end
      else do
        chars = chars + length(str)
        lines = lines + 1
        end
      end

    otherwise
      nop

    end /* select */

  if state~='start' then do
    call writeln('out',str)
    str= readln(in)
    end
  end

call close('out')
if chars+lines ~= hsize then say tempfile': article size mismatch (#! rnews' hsize '!=' chars+lines')'
if postnews(tempfile,msgid,groups) > 1 then cross= cross+1
articles= articles+1
address command 'Delete QUIET "'tempfile'"'
say 'ax-rnews: imported' articles 'article(s),' cross 'crossposting(s)'
exit

/**/

postnews: procedure expose axnews history articles;
  parse arg fname, msgid, groups
  n=0
  do while words(groups) > 0
    n=n+1
    parse var groups g ',' groups
    gname= strip(g)
    gpath= axnews || translate(gname,'/','.')
    if ~exists(gpath) then call addgroup(gname)
    call open('tmp',gpath'/.next'); artno= readln('tmp'); call close('tmp');
    address command 'Echo "'artno+1'" > "'gpath'/.next"'
    /*say fname': "'msgid'" -> "'gname'" article #'artno*/
    address command 'Copy QUIET FROM "'fname'" TO "'gpath'/'artno'"'
    address command 'Echo "'msgid || d2c(9) || translate(date(),'-',' ') time() || d2c(9) || gname'" >> "'history'"'
    end
  return n


addgroup: procedure expose axnews
  parse arg gname
  gpath= axnews || translate(gname,'/','.')
  if ~exists(gpath) then do
    call makepath(gpath)
    address command 'Echo "0" > "'gpath || '/.lowest"'
    address command 'Echo "0" > "'gpath || '/.next"'
    address command 'Echo "'gname '60" >> "'axconfig("newsgroupfile")'"'
    end
  return 0


/*@<pathonly><makepath><canexist><axconfig>*/

/* return the non-file part of a pathname */

pathonly: procedure
  parse arg path
  if (words(path) > 0) & (right(path,1) ~= ':') then do
    if right(path,1) = '/' then path= left(path,length(path)-1)
    if lastpos('/',path) > lastpos(':',path) then path= left(path,lastpos('/',path)-1)
                                             else path= left(path,lastpos(':',path))
    end
  return path



/* create all non-existant directories in a path */

makepath: procedure
  parse arg path
  if right(path,1) = '/' then path= left(path,length(path)-1)
  if ~exists(path) then do
    call makepath( pathonly(path) )
    address command 'MakeDir NAME "'path'"'
    end
  return 0


/*
 * return   1  if the device or volume name in given pathname exists
 *             or if no device or volume was present (current device)
 *          0  if the device or volume name does not exist
 */

canexist: procedure
  parse upper arg path
  if pos(':',path) < 1 then return 1 /* current device */
  call pragma('W','N')
  return exists( left(path,lastpos(':',path)) )



/* get an AXsh configuration value */

axconfig: procedure
  tempfile = "T:axconfig." || pragma('Id')
  rc_index  = "AXsh:rexx/rc.index"
  var_val=""; var_file=""; var_defval="";

  parse upper arg var_name
  if left(var_name,1) ~= '%' then var_name = '%'var_name
  if right(var_name,1) ~= ':' then var_name = var_name':'

  if open('idx',rc_index,'Read') then do
    do until (eof('idx') | (var_file~=''))
      str= translate(readln('idx'),' ',d2c(9))
      if words(str) > 0 then do
        parse var str vname ' ' fname '"' defval '"'
        if upper(vname) = var_name then do
          var_file= strip(fname,'B',' 'd2c(9))
          var_defval= defval
          end
        end
      end
    call close('idx')
    end
  else say 'Could not read "'rc_index'"'

  if words(var_file) > 0 then do
    if open('rc',var_file,'Read') then do
      do until (eof('rc') | (var_val~=''))
        str= translate(readln('rc'),' ',d2c(9))
        if upper(word(str,1)) = var_name then var_val = strip(readln('rc'),'B',' 'd2c(9))
        end
      call close('rc')
      end
    else say 'Could not examine "'var_file'" for' var_name
    end
  else do
    if words(var_defval) > 0 then var_val= var_defval
    else say 'No such config variable:' var_name
    end

  return var_val

